% ------------------------------------------------------------------------
% Simulink model for the application note solid-state transformer AN015
% Copyright : Imperix Switzerland ltd 2024
% Engineers : Jonathan Orsinger
% ------------------------------------------------------------------------

clc; close all; clearvars -except SIMULINK_MODE variant_generation variant_simulation

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   Timing parameters     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
f_clk0 = 20e3;              % [Hz], interrupt and switching frequency
Ts = 1/f_clk0;              % [s], interrupt and switching period

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   Topology parameters   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N_perPhase     = 2;    		% number of stages per phase
N_phase        = 3;	 		% number of phases (1 or 3[star])

%Phase mapping
phase_mapping  = repelem(1:N_phase, N_perPhase); % phase mapping for each stage

% Derived parameters
N = N_perPhase*N_phase; % total number of stages

% Installed stages (in model)
N_installed = 6;
if N > N_installed
    error(['The total number of stages must not exceed N_installed = ' num2str(N_installed)]);
end

% Matrix for averaging dc-link voltages in each phase
M_Vdc_mean = ones(1,N_perPhase)/N_perPhase;
if N_phase == 3
	M_Vdc_mean = blkdiag(M_Vdc_mean, M_Vdc_mean, M_Vdc_mean);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%    Power  parameters    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Grid voltage
Vac_MV_rms_LL = 400; %Vrms line-to-line
% Grid frequency
f_MV_grid = 50;
% Grid inductors
Lb = 2.5e-3;
Rb = 22e-3;
% DAB
n  = 2; % Vp = n*Vs
L1 = 47e-6;
L2 = 15e-6;
Ltot = 4*L1+4*L2*n^2;
% LV AC filter
Lf = 2.2e-3;
RLf = Rb;
Cf = 10e-6;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   Control parameters    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pre-charge state machine, safety checks
f_MV_min 	  = f_MV_grid*0.96;  % [Hz]
f_MV_max 	  = f_MV_grid*1.04;  % [Hz]
V_MV_grid_min = 50;              % [V peak]
Vdc_prim_max  = 400;             % [V dc]

% Setpoints
Vdc_prim_ref   = 380; 				% [V dc] DC capacitor voltage reference at primary
Vdc_sec_ref    = Vdc_prim_ref/n;    % [V dc] secondary DC-link voltage reference

% Grid (HV) current control (magnitude optimum)
Tdtot = 2/f_clk0; % total control delay
Tn = Lb/Rb;
Ti = 2/Rb*Tdtot;
Kp_ig = Tn/Ti;
Ki_ig = 1/Ti;

Ig_MV_ref_max = 10; %A, saturation limit and anti-windup

% Average DC-link voltage control (primary)
Cdc_st = 2*1.6e-3;            % DC-link capacitance of one stage
Cdc_eq = Cdc_st/N_perPhase;   % DC-link capacitance of equivalent non-cascaded inverter

wBW_Vdc = 0.8*pi*f_MV_grid/2; % [rad/s] bandwidth of controller
PM_Vdc  = 50;			      % [deg] phase margin of controller

a_Vdc = tan(PM_Vdc/180*pi);
boostFactor = Vdc_prim_ref/(Vac_MV_rms_LL/sqrt(N_phase)*sqrt(2)/N_perPhase);
Kp_Vdc = wBW_Vdc*2/3*sqrt(3)*boostFactor*Cdc_eq/sqrt(1+1/a_Vdc^2);
Ki_Vdc = Kp_Vdc*wBW_Vdc/a_Vdc;

% DAB voltage controller
Cdc_sec = N*5e-3+3*500e-6; % total output capacitance (secondary DC-link)
a_BW = 100; % divider for BW compared to f_clk0
a_c  = 10; % divider between BW and corner in open-loop transfer function

Kp_v_DAB = f_clk0^2*Ltot*Cdc_sec/2/Vdc_prim_max*a_c/sqrt(a_c^2+1)*2*pi/a_BW;
Ki_v_DAB = 1/a_c/a_BW*f_clk0*2*pi*Kp_v_DAB;

% DAB softstart and reference ramps
DAB_ramp_duration = 0.5;   % [s]

% LV AC current controller
Tdtot = 2/f_clk0; % total control delay
Tn = Lf/RLf;
Ti = 2/Rb*Tdtot;
Kp_Iac = Tn/Ti;
Ki_Iac = 1/Ti;

% LV AC voltage controller
Vg_LV_ref = 50*sqrt(2); %[V pk] LV voltage amplitude reference (phase to neutral)

a_Vac = 2;
Tn2 = a_Vac^2*2*Tdtot;
Ti2 = a_Vac^3*(2*Tdtot)^2/Cf;

Kp_Vac = Tn2/Ti2;
Ki_Vac = 1/Ti2;

% LV AC frequency
sync_MVLV		 = 1;  % [bool] synchronize LV with MV
f_LV 	         = 50; % [Hz]   frequency of LV grid (ignored if sync_MVLV == 1)
phaseShift_MVLV  = 0;  % [rad]  phase shift between MV and LV grid (ignored if sync_MVLV == 0)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Simulation parameters  %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial voltages
Vdc0_prim = Vdc_prim_ref;
Vdc0      = Vdc0_prim;
Vdc0_sec  = Vdc_sec_ref;

% Simulation mode (bypass the state machine)
isSim = 1; % ignored in code generation